@@ -38,19 +38,19 @@ impl AccelerationStructure {
 
     /// Query an address of a acceleration structure.
     #[doc = "[Vulkan Manual Page](https://www.khronos.org/registry/vulkan/specs/1.2-extensions/man/html/vkGetAccelerationStructureDeviceAddressKHR.html)"]
-    pub fn device_address(&self) -> vk::DeviceAddress {
+    #[cfg_attr(feature = "profiling", profiling::function)]
+    pub unsafe fn device_address(&self) -> vk::DeviceAddress {
         let info = vk::AccelerationStructureDeviceAddressInfoKHRBuilder::new()
             .acceleration_structure(self.raw);
-        unsafe {
-            self.context
-                .device
-                .get_acceleration_structure_device_address_khr(&info)
-        }
+        self.context
+            .device
+            .get_acceleration_structure_device_address_khr(&info)
     }
 
     /// Query acceleration structure meta-data on the host.
     #[doc = "[Vulkan Manual Page](https://www.khronos.org/registry/vulkan/specs/1.2-extensions/man/html/vkWriteAccelerationStructuresPropertiesKHR.html)"]
-    pub fn write_acceleration_structures_properties(
+    #[cfg_attr(feature = "profiling", profiling::function)]
+    pub unsafe fn write_acceleration_structures_properties(
         &self,
         query_type: vk::QueryType,
         data: &[u8],
@@ -58,24 +58,22 @@ impl AccelerationStructure {
     ) -> Result<()> {
         let acceleration_structures = [self.raw];
         #[allow(clippy::as_conversions)]
-        unsafe {
-            self.context
-                .device
-                .write_acceleration_structures_properties_khr(
-                    &acceleration_structures,
-                    query_type,
-                    data.len(),
-                    data.as_ptr() as *mut c_void,
-                    stride,
-                )
-        }
-        .map_err(|err| {
-            #[cfg(feature = "tracing")]
-            error!(
-                "Unable to query acceleration structure meta-data on the host: {}",
-                err
-            );
-            AscheError::VkResult(err)
-        })
+        self.context
+            .device
+            .write_acceleration_structures_properties_khr(
+                &acceleration_structures,
+                query_type,
+                data.len(),
+                data.as_ptr() as *mut c_void,
+                stride,
+            )
+            .map_err(|err| {
+                #[cfg(feature = "tracing")]
+                error!(
+                    "Unable to query acceleration structure meta-data on the host: {}",
+                    err
+                );
+                AscheError::VkResult(err)
+            })
     }
 }
